(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(['exports', 'jquery'], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require('jquery'));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.jquery);
    global.jqueryTrack = mod.exports;
  }
})(this, function (exports, _jquery) {
  'use strict';

  Object.defineProperty(exports, "__esModule", {
    value: true
  });

  var _jquery2 = _interopRequireDefault(_jquery);

  function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
      default: obj
    };
  }

  /**
   * @const {Object} Default plugin settings
   */
  var defaults = {
    debug: false,
    nonInteraction: false,
    social: false
  };

  /**
   * Helper function for debugging.
   */
  var log = function log(fields) {
    var message = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

    if (!window || !window.console) {
      return;
    }

    window.console.log('GA', 'send', fields);
  };

  /**
   * Public helper function to trigger the GA event. Also helps when you aren't
   * able to to attach data attr to DOM nodes and just need to trigger an
   * event immediately.
   *
   * NOTE: If it's a social interaction:
   *   - `eventCategory` maps to `socialNetwork`
   *   - `eventAction` maps to `socialAction`
   *   - `eventLabel` maps to `socialTarget`
   *   - `eventValue` maps to `page`
   *
   * @see {@link https://developers.google.com/analytics/devguides/collection/analyticsjs/events}
   * @see {@link https://developers.google.com/analytics/devguides/collection/analyticsjs/social-interactions}
   *
   * @param {Object}  fields Object containing GA event data.
   * @param {boolean} debug  Whether debugging is turned on.
   */
  var trigger = function trigger(fields) {
    var debug = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

    // if debug mode is on, log the ga event data and return
    // before actually triggering the event.
    if (debug) {
      log(fields);
      return;
    }

    if (fields.hitType === 'social') {
      // re-map fields object for social events
      ga('send', {
        hitType: 'social',
        socialNetwork: fields.eventCategory,
        socialAction: fields.eventAction,
        socialTarget: fields.eventLabel || null,
        page: fields.eventValue || null
      });

      return;
    }

    // trigger a standard GA event
    ga('send', fields);
  };

  /**
   * Tracking method that runs for each element in the jQuery object
   * the plugin gets called on.
   *
   * @param  {Object} settings Plugin settings
   */
  var track = function track(settings) {
    var $element = (0, _jquery2.default)(this);
    var details = _jquery2.default.parseJSON($element.attr('data-ga-event-details'));

    if (!_jquery2.default.isPlainObject(details)) {
      return;
    }

    var eventType = details.eventType || 'click';
    var isSocial = details.hitType === 'social' || settings.social === true;
    var fields = {
      hitType: isSocial ? 'social' : 'event',
      eventCategory: details.eventCategory || '',
      eventAction: details.eventAction || '',
      eventLabel: details.eventLabel || null,
      eventValue: details.eventValue || null,
      nonInteraction: details.nonInteraction || settings.nonInteraction
    };

    // if the event type is anything other than 'load', bind an event handler
    // to the element that will trigger a GA event at the time of user action.
    if (eventType !== 'load') {
      $element.on(eventType, trigger.bind(undefined, fields, settings.debug));
      return;
    }

    // trigger event immediately when an event type of load is specified
    trigger(fields, settings.debug);
  };

  /**
   * The jquery-track plugin init method.
   *
   * @see {@link http://github.com/codfish/jquery-track}
   * @see {@link http://learn.jquery.com/plugins/advanced-plugin-concepts/}
   *
   * @param  {Object} options Plugin options
   *
   * @return {jQuery} Returns the jQuery object that `track()`
   *                  was called on to allow for training.
   */
  var plugin = function plugin(options) {
    var settings = _jquery2.default.extend({}, _jquery2.default.fn.track.defaults, options);

    return this.each(function (idx, element) {
      return track.call(element, settings);
    });
  };

  // expose public objects
  plugin.trigger = trigger;
  plugin.defaults = defaults;
  _jquery2.default.fn.extend({ track: plugin });

  exports.default = plugin;
});
//# sourceMappingURL=jquery.track.js.map
